@@ -620,6 +620,7 @@ class Check(object):
         if self._exception:
             _callable(*args, **kwargs)
 
+
 def _check_default_argument(f, arg, value):
     if value is not None and arg in f.__annotations__:
         templ = f.__annotations__[arg]
@@ -628,6 +629,73 @@ def _check_default_argument(f, arg, value):
             raise EnsureError(msg.format(arg=arg, f=f, t=templ))
 
 
+class WrappedFunction:
+    """
+    Wrapper for functions to check argument annotations
+    """
+
+    def __init__(self, arg_properties, f):
+        self.arg_properties = arg_properties
+        self.f = f
+        self.__doc__ = f.__doc__
+
+    def __call__(self, *args, **kwargs):
+        for arg, templ, pos in self.arg_properties:
+            if pos is not None and len(args) > pos:
+                value = args[pos]
+            elif arg in kwargs:
+                value = kwargs[arg]
+            else:
+                continue
+
+            if not isinstance(value, templ):
+                msg = "Argument {arg} to {f} does not match annotation type {t}"
+                raise EnsureError(msg.format(arg=arg, f=self.f, t=templ))
+
+        return self.f(*args, **kwargs)
+
+    def __getattr__(self, attr_name):
+        return getattr(self.f, attr_name)
+
+    def __repr__(self):
+        return repr(self.f)
+
+    def __str__(self):
+        return str(self.f)
+
+    def __get__(self, obj, type=None):
+        return types.MethodType(self, obj)
+
+
+class WrappedFunctionReturn(WrappedFunction):
+    """
+    Wrapper for functions to check argument annotations with return checking
+    """
+
+    def __init__(self, arg_properties, f, return_templ):
+        super().__init__(arg_properties, f)
+        self.return_templ = return_templ
+
+    def __call__(self, *args, **kwargs):
+        for arg, templ, pos in self.arg_properties:
+            if pos is not None and len(args) > pos:
+                value = args[pos]
+            elif arg in kwargs:
+                value = kwargs[arg]
+            else:
+                continue
+
+            if not isinstance(value, templ):
+                msg = "Argument {arg} to {f} does not match annotation type {t}"
+                raise EnsureError(msg.format(arg=arg, f=self.f, t=templ))
+
+        return_val = self.f(*args, **kwargs)
+        if not isinstance(return_val, self.return_templ):
+                msg = "Return value of {f} does not match annotation type {t}"
+                raise EnsureError(msg.format(f=self.f, t=self.return_templ))
+        return return_val
+
+
 def ensure_annotations(f):
     """
     Decorator to be used on functions with annotations. Runs type checks to enforce annotations. Raises
@@ -650,6 +718,7 @@ def ensure_annotations(f):
 
         >>> ensure.EnsureError: Argument y to <function f at 0x109b7c710> does not match annotation type <class 'float'>
     """
+
     if f.__defaults__:
         for rpos, value in enumerate(f.__defaults__):
             pos = f.__code__.co_argcount - len(f.__defaults__) + rpos
@@ -669,34 +738,13 @@ def ensure_annotations(f):
                 arg_properties.append((arg, templ, None))
             else:
                 arg_properties.append((arg, templ, pos))
-    from functools import wraps
 
-    has_return_annotation = False
     if 'return' in f.__annotations__:
-        has_return_annotation = True
         return_templ = f.__annotations__['return']
+        return WrappedFunctionReturn(arg_properties, f, return_templ)
+    else:
+        return WrappedFunction(arg_properties, f)
 
-    @wraps(f)
-    def wrapper(*args, **kwargs):
-        for arg, templ, pos in arg_properties:
-            if pos is not None and len(args) > pos:
-                value = args[pos]
-            elif arg in kwargs:
-                value = kwargs[arg]
-            else:
-                continue
-
-            if not isinstance(value, templ):
-                msg = "Argument {arg} to {f} does not match annotation type {t}"
-                raise EnsureError(msg.format(arg=arg, f=f, t=templ))
-
-        return_val = f(*args, **kwargs)
-        if has_return_annotation:
-            if not isinstance(return_val, return_templ):
-                msg = "Return value of {f} does not match annotation type {t}"
-                raise EnsureError(msg.format(f=f, t=return_templ))
-        return return_val
-    return wrapper
 
 ensure = Ensure()
 check = Check()
