@@ -138,7 +138,7 @@ class TestEnsure(unittest.TestCase):
         f_code = """
 from ensure import ensure_annotations
 
-global f, g
+global f, g, h
 
 @ensure_annotations
 def f(x: int, y: float) -> float:
@@ -148,20 +148,26 @@ def f(x: int, y: float) -> float:
 
 @ensure_annotations
 def g(x: str, y: str="default") -> str:
+    '''Simply add numbers together'''
     t = x+y
     return t
+
+@ensure_annotations
+def h(x: str, y: int):
+    '''Does some work'''
+    return x * y
 """
         exec(f_code)
         self.assertEqual(f(1, 2.3), 3.3)
         self.assertEqual(f(1, y=2.3), 3.3)
         self.assertEqual(f(y=1.2, x=3), 4.2)
-        with self.assertRaisesRegexp(EnsureError, "Argument y to <function f at .+> does not match annotation type <class 'float'>"):
+        with self.assertRaisesRegex(EnsureError, "Argument y to <function f at .+> does not match annotation type <class 'float'>"):
             self.assertEqual(f(1, 2), 3.3)
-        with self.assertRaisesRegexp(EnsureError, "Argument y to <function f at .+> does not match annotation type <class 'float'>"):
+        with self.assertRaisesRegex(EnsureError, "Argument y to <function f at .+> does not match annotation type <class 'float'>"):
             self.assertEqual(f(y=2, x=1), 3.3)
-        with self.assertRaisesRegexp(EnsureError, "Return value of <function f at .+> does not match annotation type <class 'float'>"):
+        with self.assertRaisesRegex(EnsureError, "Return value of <function f at .+> does not match annotation type <class 'float'>"):
             self.assertEqual(f(1, -2.3), 4)
-        with self.assertRaisesRegexp(EnsureError, "Return value of <function f at .+> does not match annotation type <class 'float'>"):
+        with self.assertRaisesRegex(EnsureError, "Return value of <function f at .+> does not match annotation type <class 'float'>"):
             self.assertEqual(f(x=1, y=-2.3), 4)
 
         # Test with mixtures of keyword and positional args
@@ -169,6 +175,13 @@ def g(x: str, y: str="default") -> str:
         self.assertEqual(g("the ", "bomb"), "the bomb")
         self.assertEqual(g(y=g("the ", y="bomb"), x="somebody set up us "), "somebody set up us the bomb")
 
+        self.assertEqual('g', g.__name__)
+        self.assertEqual('Simply add numbers together', g.__doc__)
+        self.assertRegex(repr(g), '<function g at 0x[0-9a-f]+>')
+        self.assertEqual('h', h.__name__)
+        self.assertEqual('Does some work', h.__doc__)
+        self.assertRegex(repr(h), '<function h at 0x[0-9a-f]+>')
+
     @unittest.skipIf(sys.version_info < (3, 0), "Skipping test that requires Python 3 features")
     def test_annotations_with_bad_default(self):
         f_code = """
@@ -184,7 +197,7 @@ def f(x: int, y: float=None) -> float:
 def g(x: str, y: str=5, z='untyped with default') -> str:
     return x+y+str(z)
 """
-        with self.assertRaisesRegexp(EnsureError, "Default argument y to <function g at .+> does not match annotation type <class 'str'>"):
+        with self.assertRaisesRegex(EnsureError, "Default argument y to <function g at .+> does not match annotation type <class 'str'>"):
             exec(f_code)
         # Make sure f still works as None should be excluded from default test
         self.assertEqual(f(1, 2.3), 3.3)
@@ -211,7 +224,7 @@ def f(x: int, y: float, *args, z: int=5) -> float:
         self.assertEqual(2.0, f(3, 4.0))
         self.assertEqual(62.0, f(3, 4.0, 10, 20, 30))
         self.assertEqual(66.0, f(3, 4.0, 10, 20, 30, z=1))
-        with self.assertRaisesRegexp(EnsureError, "Argument z to <function f at .+> does not match annotation type <class 'int'>"):
+        with self.assertRaisesRegex(EnsureError, "Argument z to <function f at .+> does not match annotation type <class 'int'>"):
             self.assertEqual(66.0, f(3, 4.0, 10, 20, 30, z='hello world'))
 
     @unittest.skipIf(sys.version_info < (3, 0), "Skipping test that requires Python 3 features")
@@ -237,7 +250,7 @@ def f(x: int, y: float, *args, z: int=5) -> str:
         self.assertEqual('2.0abc', f(3, 4.0, 'abc'))
         self.assertEqual('2.0abc2.0def', f(3, 4.0, 'abc', 'def'))
         self.assertEqual('3.0abc3.0def', f(3, 4.0, 'abc', 'def', z=4))
-        with self.assertRaisesRegexp(EnsureError, "Argument z to <function f at .+> does not match annotation type <class 'int'>"):
+        with self.assertRaisesRegex(EnsureError, "Argument z to <function f at .+> does not match annotation type <class 'int'>"):
             self.assertEqual('3.0abc3.0def', f(3, 4.0, 'abc', 'def', z='school'))
 
     @unittest.skipIf(sys.version_info < (3, 0), "Skipping test that requires Python 3 features")
@@ -256,8 +269,38 @@ def f(x: int, y: float, *args, z: int='not an int') -> str:
 
     return r
 """
-        with self.assertRaisesRegexp(EnsureError, "Default argument z to <function f at .+> does not match annotation type <class 'int'>"):
+        with self.assertRaisesRegex(EnsureError, "Default argument z to <function f at .+> does not match annotation type <class 'int'>"):
             exec(f_code)
 
+    @unittest.skipIf(sys.version_info < (3, 0), "Skipping test that requires Python 3 features")
+    def test_annotations_on_bound_methods(self):
+        f_code = """
+from ensure import ensure_annotations
+
+global C
+
+class C(object):
+    @ensure_annotations
+    def f(self, x: int, y: float) -> str:
+        return str(x+y)
+
+    @ensure_annotations
+    def g(self, x: int, y: float):
+        return str(x+y)
+
+"""
+        exec(f_code)
+        c = C()
+        self.assertEqual('3.3', c.f(1, 2.3))
+        self.assertRegex(repr(c.f), '<bound method C.f of <__main__.C object at 0x[0-9a-f]+>>')
+        with self.assertRaisesRegex(EnsureError, "Argument x to <function (C.f|f) at .+> does not match annotation type <class 'int'>"):
+            g = C().f(3.2, 1)
+
+        self.assertEqual('3.3', c.g(1, 2.3))
+        self.assertRegex(repr(c.g), '<bound method C.g of <__main__.C object at 0x[0-9a-f]+>>')
+        with self.assertRaisesRegex(EnsureError, "Argument x to <function (C.g|g) at .+> does not match annotation type <class 'int'>"):
+            g = C().g(3.2, 1)
+
+
 if __name__ == '__main__':
     unittest.main()
